#!/usr/bin/perl

# creates a histogram of the x positions of the particles and 
# pipes it gnuplot for plotting
use strict;

my $anzArgs = @ARGV;
if ($anzArgs != 4) {
	die "Usage: $0   Xmin  Xmax  Nbin  outSteps\n";
}

# print STDERR "Full command name = $0\n";

# range of the histogram
my $XMIN = shift;
my $XMAX = shift;
my $NBINS = shift;
# average over $ANZOUT steps
my $ANZOUT = shift;

my $dx = ($XMAX - $XMIN) / $NBINS;

print STDERR "$NBINS bins of size $dx between $XMIN and $XMAX\n";

my $isTimeLine = 1;
my $zeitPunkt;
my $anzProt;

# Arrays for the densities
my @xhist_na;
my @xhist_cl;
for(my $i=0; $i<$NBINS; $i++) {
	$xhist_na[$i] = 0;
	$xhist_cl[$i] = 0;
}

my $anzSchritte = 0;
my $allSchritte = 0;
my $ausSchritte = 1;

select(STDOUT); $| = 1;
print "set data style histeps\n";

while(<STDIN>)
{
	split(' ');

	if($isTimeLine == 1)
	{
		$zeitPunkt = $_[1];
		$anzProt = $_[2];
		
		$anzSchritte++;
		
		$isTimeLine = 0 if ($anzProt > 0);
		
		# print "$anzSchritte: Zeit = $zeitPunkt  Anzahl = $anzProt\n";
		
		# print 1.0*$zeitPunkt;
		
		if($ausSchritte >= $ANZOUT)
		{
			my $x = $dx / 2.0 + $XMIN;
			open(RAUS, ">histx_na.dat");
			for(my $i=0; $i<$NBINS; $i++)
			{
				print RAUS "$x\t".$xhist_na[$i]/$dx/$ANZOUT."\n";
				$x += $dx;
			}
			print RAUS "\n";
			close(RAUS);
			
			$x = $dx / 2.0 + $XMIN;
			open(RAUS, ">histx_cl.dat");
			for(my $i=0; $i<$NBINS; $i++)
			{
				print RAUS "$x\t".$xhist_cl[$i]/$dx/$allSchritte."\n";
				$x += $dx;
			}
			print RAUS "\n";
			close(RAUS);

			print "plot \[$XMIN:$XMAX\]\[0:1.2\] \"histx_na.dat\" t \"temp\", \"histx_cl.dat\" t \"All\", 0.64\n";
			
			for(my $i=0; $i<$NBINS; $i++) {
				$xhist_na[$i] = 0;
			}
			
			$ausSchritte = 0;
		}
		$ausSchritte++;
		$allSchritte++;
		
	} 
	else 
	{
		my $label = $_[0];
		my $num = $_[1];
		my $x = $_[3];
		my $y = $_[4];
		my $z = $_[5];
		
		my $ix = int(($x-$XMIN) / $dx);

		if($label =~ "Punkt")
		{
			$xhist_na[$ix]++;
			$xhist_cl[$ix]++;
		} 
		
		$anzProt--;	
		$isTimeLine = 1 if ($anzProt==0);
	}
	
}


exit;