// This file is part of the brownmove simulation package
// 
// (C) 2003 - 2009 Tihamer Geyer
// tihamer.geyer@bioinformatik.uni-saarland.de
// 
// Check for Updates at the Project Homepage 
// http://service.bioinformatik.uni-saarland.de/brownmove

#ifndef HBONDSHAPE_H
#define HBONDSHAPE_H

#include <string>
#include <vector>
#include <iostream>
#include <string>
#include <sstream>

#include "types.h"
#include "constants.h"
#include "vectormath.h"

#include "shape.h"


namespace brown{
    /**
     *@author tg
     */
    
	//! HBondShape models hydrogen bonds between donors and acceptors.
    class HBondShape : public Shape {
	
	public:
	HBondShape();
	HBondShape(const HBondShape &other);
	~HBondShape();
	
	
	/** restores the parts' (absolute and relative) positions from fullPosition */
	virtual void recalculate(const fullPosition &p);
	
	/** returns some string for testing */
	virtual std::string describe();
	
	//! adds another hookup point with position and scaling to the vector.
	//! Scale > 0 means donor, <0 acceptor. Standard donor/acceptor have |scale| = 1. 
	//! Also calculates the relative position of the charge from the molecules center given
	void addHook(hbondPos hp, const position &moleculeCenter);

	
	//! calculate the forces and torques from the hooks of this and the other HBondShape
	void calculateImpact(fullForce &myForce, fullForce &otherForce, HBondShape *es, const bool isPeriodic[3], const doublevec &cellSize);
	
	//! returns the number of hooks
	int size() { return(hooks.size()); }
	
	//! Dumps the known infos about charges in browndef format
	void dumpStructure();

	
	private:
	//! The relative positions of the charges and their charge
	vector<hbondPos> hooks;
	//! Keep track of the absolute positions of the hooks, updated when shape is moved
	vector<hbondPos> actual_hooks;
	
    };
}

#endif
