// This file is part of the brownmove simulation package
// 
// (C) 2003 - 2009 Tihamer Geyer
// tihamer.geyer@bioinformatik.uni-saarland.de
// 
// Check for Updates at the Project Homepage 
// http://service.bioinformatik.uni-saarland.de/brownmove

#include <iostream>
#include <fstream>
#include <math.h>

#include "constants.h"
#include "types.h"
#include "vectormath.h"
#include "random.h"

#include "cup.h"
#include "wall.h"
#include "molecule.h"


#include "proteinparser.h"


using namespace brown;


void VMDHeader()
{
	std::cout << 
	"mol delete all\n" <<
	"mol load graphics gestalt\n" <<
	"mol load graphics vdw\n" <<
	"mol load graphics estat\n" <<
	"mol load graphics hbond\n" <<
	"mol load graphics hi\n" <<
	"mol load graphics bond\n" <<
	"mol load graphics vdw_label\n" <<
	"mol load graphics estat_label\n" <<
	"mol load graphics hbond_label\n" <<
	"mol load graphics hi_label\n" <<
	"mol load graphics bond_label\n" <<
	"mol load graphics timeline\n" <<
	"mol load graphics ramachandran\n" <<
	"graphics 6 materials off\n" <<
	"graphics 7 materials off\n" <<
	"graphics 8 materials off\n" <<
	"graphics 9 materials off\n" <<
	"graphics 10 materials off\n" <<
	"graphics 11 materials off\n" <<
	"graphics 12 materials off\n" <<
	"material add MSc1\n" <<
	"material change ambient MSc1 0.00\n" <<
	"material change specular MSc1 0.00\n" <<
	"material change diffuse MSc1 0.85\n" <<
	"material change shininess MSc1 0.53\n" <<
	"material change opacity MSc1 1.00\n" <<
	"graphics 0 material MSc1\n" <<
	"graphics 1 material MSc1\n" <<
	"graphics 2 material MSc1\n" <<
	"graphics 4 material MSc1\n" <<
	"graphics 3 material MSc1\n" <<
	"graphics 5 material MSc1\n" <<
	"graphics 0 color orange\n" <<
	"graphics 1 color yellow\n" <<
	"graphics 4 color silver\n" <<
	"graphics 5 color gray\n" <<
	"axes location off\n" <<
	"mol off 0\n" <<
	"mol off 3\n" <<
	"mol off 4\n" <<
	"mol off 10\n" <<
	"mol off 8\n" <<
	"mol off 6\n" <<
	"mol off 9\n" <<
	"mol off 7\n" <<
	"display projection orthographic\n" << std::endl;
}


void VMDBreak() {
	std::cout << "display resetview\n" <<
	"graphics 0 delete all\n" <<
	"graphics 1 delete all\n" <<
	"graphics 2 delete all\n" <<
	"graphics 3 delete all\n" <<
	"graphics 4 delete all\n" <<
	"graphics 5 delete all\n" <<
	"graphics 6 delete all\n" <<
	"graphics 7 delete all\n" <<
	"graphics 8 delete all\n" <<
	"graphics 9 delete all\n" <<
	"graphics 10 delete all\n" <<
	"graphics 11 delete all\n" <<
	"graphics 12 delete all\n" <<
	"graphics 6 materials off\n" <<
	"graphics 7 materials off\n" <<
	"graphics 8 materials off\n" <<
	"graphics 9 materials off\n" <<
	"graphics 10 materials off\n" <<
	"graphics 11 materials off\n" <<
	"graphics 12 materials off\n" <<
	"graphics 0 material MSc1\n" <<
	"graphics 1 material MSc1\n" <<
	"graphics 2 material MSc1\n" <<
	"graphics 4 material MSc1\n" <<
	"graphics 3 material MSc1\n" <<
	"graphics 5 material MSc1\n" <<
	"graphics 0 color orange\n" <<
	"graphics 1 color yellow\n" <<
	"graphics 4 color silver\n" <<
	"graphics 5 color gray\n" <<
	"mol off 0\n" <<
	"mol off 3\n" <<
	"mol off 4\n" <<
	"mol off 10\n" <<
	"mol off 8\n" <<
	"mol off 6\n" <<
	"mol off 9\n" <<
	"mol off 7\n\n" << std::endl;
}


void VMDFooter() {
	std::cout << "display resetview\n" << std::endl;
}



void printHooks(std::vector<bondPos> &haken) 
{
	std::vector<bondPos>::iterator it1, it2;
	// int i = 1;
	
	for(it1 = haken.begin(); it1 != haken.end(); ++it1) {
		for(it2 = it1; it2 != haken.end(); ++it2) {
			if((it1 != it2) && (*it1).idx == (*it2).idx) {
				std::cout << "graphics 5 cylinder {" <<10.0*(*it1).pos[0]<<" "<<10.0*(*it1).pos[1]<<" "<<10.0*(*it1).pos[2]<<"} {"<<10.0*(*it2).pos[0]<<" "<<10.0*(*it2).pos[1]<<" "<<10.0*(*it2).pos[2]<<"} radius 0.3 resolution 28" << std::endl;
			}
		}
	}
}




// Read next line, chop off leading whitespace and ignore comment lines
int myGetLine(FILE *fp_in, char *newLine, const int maxLen)
{
	int c;
	int count = 0;
	bool leadingWhite = true;
	bool ignoreLine = true;
	
	while(ignoreLine) {
		leadingWhite = true;
		count = 0;
		
		c = fgetc(fp_in);
		
		while(c != '\n') {
			if(c == EOF) {
				return(c); }
			
			if((c != ' ' && c != '\t')  || leadingWhite == false) {
				leadingWhite = false;
				newLine[count] = c;
				count++;
				if(count >= maxLen) {
					newLine[count-1] = '\0';
					return(0);
				}
			}
			c = fgetc(fp_in);
		}
		newLine[count] = '\0';
		if(count > 0 && newLine[0] != '#') {
			ignoreLine = false; }
	}
	return(0);
}



int firstWords(const char *newLine, char *tag, char *tag2)
{
	int lang = strlen(newLine);
	int i = 0;

	if(sscanf(newLine, "%s %s", tag, tag2) == 2) {
		return(2); }
	else if(sscanf(newLine, "%s", tag) == 1) {
		tag2[0] = '\0';		// return empty string as second arg
		return(1); } 
	else {
		return(0);
	}
}


int main(int argc, char *argv[])
{
	if(argc != 3) {
		cerr << "Extractor 1.0: read brownmove output and dump vdw shapes of the given protein" << endl;
		cerr << "   compiled" <<
		#include "compileInfo.h"
		<< endl;
		cerr << "\n  Usage:  " << argv[0] << "  <protein setup file>  <protein number (>= 1)>" << endl;
		return(-1);
	}
	
	// init parser and build template molecule
	double timeStep = 0.0;								// this is a dummy variable
	ProteinParser *p = new ProteinParser(timeStep);
	
	Molecule *m1 = new Molecule("temp", 0);

	int molID, vdWMaxCol=1000;
	// define a single externalForce of type NONE => indicates to proteinParser to ignore all externalShape definitions
	// (actually, links them to a filed of type NONE)
	list<externalForce> felder;
	externalForce xf;
	felder.push_back(xf);
	
	sscanf(argv[2], "%d", &molID);
	if(!(p->parse(m1, argv[1], (molID-1), vdWMaxCol, felder))) {
		std::cerr << "main::init_cup: error parsing molecule definition " << molID << " in file >>" << argv[1] << std::endl;
		exit(-1);
	}
	std::cerr << "building template molecule " << (m1->getid()).mclass << " done" << std::endl;
	

	char newLine[1024], tag1[1024], tag2[1024], protLabel[1024], gestLabel[1024], prefix[1024];
	double zeitPunkt, dummy;
	long int idx;
	fullPosition fp;
	Gestalt *gestP;
	std::vector<bondPos> haken;
	position CM;
	
	VMDHeader();
	
	// loop: read stdin until EOF
	while(myGetLine(stdin, newLine, 1023) != EOF)
	{
		std::cerr << "Read: " << newLine << std::endl;
		
		// check for "timeline"
		firstWords(newLine, tag1, tag2);
		
		// std::cerr << "  tag1 = >>" << tag1 << "<<  tag2 = >>" << tag2 << "<<" << std::endl;
		
		if(strcmp(tag1, "timeline") == 0) {
			sscanf(tag2, "%lf", &zeitPunkt);
			
			// output of bonds
			if(haken.size() > 1) {
				printHooks(haken);
				VMDBreak();
			}
			
			std::cerr << "\n\n  zeitPunkt = " << zeitPunkt << std::endl;
			haken.clear();
		}
		else
		{
			// extract position
			if(sscanf(newLine, "%s %ld %s %lf %lf %lf %lf %lf %lf %lf",
				   protLabel, &idx, gestLabel, &fp.pos[0], &fp.pos[1], &fp.pos[2], &fp.rot.phi, &fp.rot.n[0], &fp.rot.n[1], &fp.rot.n[2]) != 10)
			{
				std::cerr << "Extractor: error extracting position from line >>" << newLine << "<<" << std::endl;
				exit(-1);
			}
		
			if(string(protLabel) == (m1->getid()).mclass)
			{
				doublevec_set(CM, m1->getCenterOfMass());
				
				std::cerr << "Checking protein " << protLabel << ", Gestalt " << gestLabel << " at t = " << zeitPunkt << std::endl;
				// apply shift and rotation to gestalt of template molecule
				gestP = m1->getGestalt(gestLabel);
				std::cerr << "  identified Gestalt " << gestP->getLabel() << std::endl;
				// doublevec_sub(fp.pos, CM);
				
				gestP->recalculate(fp);
				
				// dump vdw spheres to stdout
				sprintf(prefix, "%lf %s %s", zeitPunkt, protLabel, gestLabel);
				std::cerr << "  prefix = " << prefix << std::endl;
				// gestP->getvdWShape()->dumpSpheres(prefix);
				gestP->getvdWShape()->dumpVMD(prefix);
				
				gestP->getBondShape()->listHooks(haken);
				
				std::cerr << "Extractor: haken.size() = " << haken.size() << std::endl;
			}
	
		}
		
	}
	// output of bonds
	if(haken.size() > 1) {
		printHooks(haken);
	}
	VMDFooter();
	
	delete m1;
	
	return (0);
}
